'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _dataloader = require('dataloader');

var _dataloader2 = _interopRequireDefault(_dataloader);

var _lodash = require('lodash');

var _node = require('./../node');

var _queryBuilder = require('../queryBuilder');

var _loaderFilter = require('./loaderFilter');

var _loaderFilter2 = _interopRequireDefault(_loaderFilter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Model loader use Dataloader
 * to batch and cache DB calls
 * using PK or indexed fields
 */

var ModelLoader = function () {
  function ModelLoader(model) {
    _classCallCheck(this, ModelLoader);

    this.model = model;
    this.modelName = model._schema._model._name;
    this._loaders = {};
  }

  /**
   * Load by Id
   * @param modelId
   * @returns {IgnoredPaths|Object|*|void}
   */


  _createClass(ModelLoader, [{
    key: 'loadById',
    value: function loadById(modelId) {
      return this._getOrCreateLoader('loadBy', 'id').load(modelId).then(function (results) {
        return new _loaderFilter2.default(results);
      });
    }

    /**
     * Load by any field Name
     * @param fieldName
     * @param value
     * @returns {IgnoredPaths|Object|*|void}
     */

  }, {
    key: 'loadBy',
    value: function loadBy(fieldName, value) {
      return this._getOrCreateLoader('loadBy', fieldName).load(value).then(function (results) {
        return new _loaderFilter2.default(results);
      });
    }

    /**
     * Related loader
     * @param relationName
     * @param FKID
     * @returns {IgnoredPaths|Object|*|void}
     */

  }, {
    key: 'related',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(relationName, FKID, options) {
        var params, filterOpt;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                params = this._batchParamsKeys(options);
                _context.next = 3;
                return this._getOrCreateLoader('query:id', relationName, this._queryRelationLoader).load(params);

              case 3:
                filterOpt = _context.sent;
                _context.next = 6;
                return this._getOrCreateLoader('relation:' + JSON.stringify(filterOpt), relationName, this._queryRelations(relationName, 'id', filterOpt)).load(FKID).then(function (results) {
                  return new _loaderFilter2.default(results, options);
                });

              case 6:
                return _context.abrupt('return', _context.sent);

              case 7:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function related(_x, _x2, _x3) {
        return ref.apply(this, arguments);
      }

      return related;
    }()
  }, {
    key: 'relatedByField',


    /**
     * Related by field name
     * @param relationName
     * @param fieldName
     * @param FKID
     * @returns {IgnoredPaths|Object|*|void}
     */
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(relationName, fieldName, FKID, options) {
        var params, filterOpt;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                params = this._batchParamsKeys(options);
                _context2.next = 3;
                return this._getOrCreateLoader('query:' + fieldName, relationName, this._queryRelationLoader).load(params);

              case 3:
                filterOpt = _context2.sent;
                return _context2.abrupt('return', this._getOrCreateLoader('relation:' + JSON.stringify(filterOpt), relationName, this._queryRelations(relationName, fieldName)).load(FKID).then(function (results) {
                  return new _loaderFilter2.default(results);
                }));

              case 5:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function relatedByField(_x4, _x5, _x6, _x7) {
        return ref.apply(this, arguments);
      }

      return relatedByField;
    }()

    /**
     * Get or create a Dataloader for this Model
     * @param loaderPrefix
     * @param fieldName
     * @param loaderFn
     * @returns {*}
     * @private
     */

  }, {
    key: '_getOrCreateLoader',
    value: function _getOrCreateLoader(loaderPrefix, fieldName, loaderFn) {
      var loaderOpt = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

      var loaderName = loaderPrefix + ':' + fieldName;
      var checkLoader = this._getLoader(loaderName);
      if (checkLoader) {
        return checkLoader;
      }
      var newloaderName = this.modelName + ':' + loaderName;
      var dataloaderFn = loaderFn || this._queryByField(fieldName);
      var Loader = new _dataloader2.default(dataloaderFn, loaderOpt);
      this._loaders[newloaderName] = Loader;
      return Loader;
    }

    /**
     * Batch param keys
     * @param filterQuery
     * @param filter
     * @param attributes
     * @param rest
     * @returns {*}
     * @private
     */

  }, {
    key: '_batchParamsKeys',
    value: function _batchParamsKeys(_ref) {
      var filterQuery = _ref.filterQuery;
      var filter = _ref.filter;
      var attributes = _ref.attributes;

      var rest = _objectWithoutProperties(_ref, ['filterQuery', 'filter', 'attributes']);

      if (filterQuery) {
        rest.filter = filter;
        rest.attributes = attributes;
      } else {
        rest.filter = {};
        rest.attributes = [];
      }
      return Object.keys(rest).sort().reduce(function (obj, objKey) {
        obj[objKey] = rest[objKey];
        return obj;
      }, {});
    }

    /**
     * Query relation Loader
     * @param queryParams
     * @returns {*}
     * @private
     */

  }, {
    key: '_queryRelationLoader',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee3(queryParams) {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                return _context3.abrupt('return', queryParams.map(function (params) {
                  var same = queryParams.filter(function (qp) {
                    return JSON.stringify(params.filter) == JSON.stringify(qp.filter) && params.index == qp.index;
                  });

                  if (same.length > 0) {
                    return (0, _lodash.maxBy)(same, 'offset');
                  }
                  return params;
                }));

              case 1:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function _queryRelationLoader(_x9) {
        return ref.apply(this, arguments);
      }

      return _queryRelationLoader;
    }()

    /**
     * Query model by any field
     * @param fieldName
     * @returns {function(*=)}
     * @private
     */

  }, {
    key: '_queryByField',
    value: function _queryByField(fieldName) {
      var _this = this;

      return function (fieldNames) {
        var r = _this.model._thinky.r;
        return _this.model.getAll(r.args((0, _lodash.uniq)(fieldNames)), {
          index: fieldName
        }).run().then(function (results) {
          return _this._mapResults(fieldNames, results, fieldName);
        });
      };
    }

    /**
     * Query relations
     * @param relationName
     * @returns {function(*)}
     * @private
     */

  }, {
    key: '_queryRelations',
    value: function _queryRelations(relationName, fieldName, nodeOpts) {
      var _this2 = this;

      return function () {
        var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee5(FkIds) {
          var thinky, r, query, results, resultSet, _ret;

          return regeneratorRuntime.wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  thinky = _this2.model._thinky;
                  r = thinky.r;
                  query = _this2.model.getAll(r.args((0, _lodash.uniq)(FkIds)), { index: fieldName }).withFields(fieldName).getJoin(_defineProperty({}, relationName, {
                    _apply: function _apply(seq) {
                      return (0, _queryBuilder.buildQuery)(seq, nodeOpts, thinky);
                    }
                  }));
                  _context5.next = 5;
                  return query.run();

                case 5:
                  results = _context5.sent;
                  resultSet = _this2._mapResults(FkIds, results, fieldName, relationName);

                  // If we need to count result
                  // I'll do another query

                  if (!nodeOpts.count) {
                    _context5.next = 12;
                    break;
                  }

                  return _context5.delegateYield(regeneratorRuntime.mark(function _callee4() {
                    var relation, FK, countResults;
                    return regeneratorRuntime.wrap(function _callee4$(_context4) {
                      while (1) {
                        switch (_context4.prev = _context4.next) {
                          case 0:
                            relation = _this2.model._joins[relationName];
                            FK = relation.rightKey;
                            _context4.next = 4;
                            return _this2._countRelated((0, _lodash.flatten)(resultSet.map(function (row) {
                              return row.map(function (innerRow) {
                                return innerRow[FK];
                              });
                            })), relationName, nodeOpts);

                          case 4:
                            countResults = _context4.sent;
                            return _context4.abrupt('return', {
                              v: (0, _queryBuilder.mapCountToResultSet)(resultSet, countResults, FK)
                            });

                          case 6:
                          case 'end':
                            return _context4.stop();
                        }
                      }
                    }, _callee4, _this2);
                  })(), 't0', 9);

                case 9:
                  _ret = _context5.t0;

                  if (!((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object")) {
                    _context5.next = 12;
                    break;
                  }

                  return _context5.abrupt('return', _ret.v);

                case 12:
                  return _context5.abrupt('return', resultSet);

                case 13:
                case 'end':
                  return _context5.stop();
              }
            }
          }, _callee5, _this2);
        }));

        return function (_x10) {
          return ref.apply(this, arguments);
        };
      }();
    }

    /**
     * Count related result set
     * @param relatedIds
     * @param relationName
     * @param opts
     * @returns {*|void|Promise}
     * @private
     */

  }, {
    key: '_countRelated',
    value: function _countRelated(relatedIds, relationName, opts) {
      var relation = this.model._joins[relationName];
      var FK = relation.rightKey;

      return (0, _queryBuilder.buildCount)(relation.model, relatedIds, FK, opts);
    }

    /**
     * Map Dataloader results
     * @param ids
     * @param results
     * @param fieldName
     * @param relationName
     * @returns {*}
     * @private
     */

  }, {
    key: '_mapResults',
    value: function _mapResults(ids, results, fieldName, relationName) {
      return ids.map(function (fkId) {
        var resultMatch = (0, _lodash.find)(results, _defineProperty({}, fieldName, fkId));
        if (resultMatch) {
          if (relationName) {
            return resultMatch[relationName];
          }
          return resultMatch;
        }
        return {};
      });
    }

    /**
     * Get a loader
     * @param loaderName
     * @returns {*}
     * @private
     */

  }, {
    key: '_getLoader',
    value: function _getLoader(loaderName) {
      return this._loaders[this.modelName + ':' + loaderName];
    }

    /**
     * Get loaders
     * @returns {{}|*}
     * @private
     */

  }, {
    key: '_getLoaders',
    value: function _getLoaders() {
      return this._loaders;
    }
  }]);

  return ModelLoader;
}();

exports.default = ModelLoader;