'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _graphqlRelay = require('graphql-relay');

var _resolver = require('./resolver');

var _resolver2 = _interopRequireDefault(_resolver);

var _typeMapper = require('./typeMapper');

var _typeMapper2 = _interopRequireDefault(_typeMapper);

var _modelLoader = require('./dataloader/modelLoader');

var _modelLoader2 = _interopRequireDefault(_modelLoader);

var _loaderFilter = require('./dataloader/loaderFilter');

var _loaderFilter2 = _interopRequireDefault(_loaderFilter);

var _nodeDefinition = require('./relay/nodeDefinition');

var _modelToGqlObjectType = require('./modelToGqlObjectType');

var _modelToGqlObjectType2 = _interopRequireDefault(_modelToGqlObjectType);

var _node = require('./node');

var _node2 = _interopRequireDefault(_node);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var defaultOptions = {
  maxLimit: 50,
  nestingLimit: 10
};

/**
 * GraphqlThinky
 */

var GraphqlThinky = function () {

  /**
   * Graphus constructor accept a Thinky
   * Instance
   *
   * @param thinky
   * @param options
   */

  function GraphqlThinky(thinky) {
    var _this = this;

    var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    _classCallCheck(this, GraphqlThinky);

    this.resolve = function (modelName, related) {
      var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      var Node = _this.node(modelName, related);
      return (0, _resolver2.default)(Node, _extends({}, _this.options, opts));
    };

    this.connect = function (modelName, related) {
      var _ref = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      var connection = _ref.connection;
      var args = _ref.args;

      var opts = _objectWithoutProperties(_ref, ['connection', 'args']);

      /*eslint-disable */
      if (!connection) throw Error('Please provide a connection option.');
      if (!connection.name) throw Error('Please provide a name for the connection based on Model: ' + modelName + '.');
      if (!connection.type) throw Error('Please provide a type for the connection based on Model: ' + modelName + '.');
      /*eslint-enable */

      var NodeConnector = _this.node(modelName, related, _extends({}, opts, {
        connection: connection
      })).connect(_extends({}, _this.options, opts));

      return {
        type: NodeConnector.connectionType,
        args: _extends({}, NodeConnector.connectionArgs, args || {}),
        resolve: NodeConnector.resolve
      };
    };

    this.connectTypeDefinition = function (gqlType, args) {
      var _connectionDefinition = (0, _graphqlRelay.connectionDefinitions)({ nodeType: gqlType });

      var connectionType = _connectionDefinition.connectionType;

      return {
        type: connectionType,
        args: _extends({}, _graphqlRelay.connectionArgs, args)
      };
    };

    this.shape = function (result) {
      return new _loaderFilter2.default(result);
    };

    this.modelToGraphQLDefinition = function (model) {
      var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (typeof model === 'string') {
        model = _this.thinky.models[model];
      }

      if (!model) {
        throw new Error('Model name: ' + model + ' not found.');
      }

      return (0, _typeMapper.toGraphQLDefinition)(model, opts);
    };

    this.createModelType = function (model) {
      var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (typeof model === 'string') {
        model = _this.thinky.models[model];
      }

      if (!model) {
        throw new Error('Model name: ' + model + ' not found.');
      }

      // Add node interface if relay is specified
      if (opts.globalId === true) {
        opts.interfaces = [_this.nodeInterface];
      }

      var gqlObjectType = (0, _modelToGqlObjectType2.default)(model, opts);

      _this.nodeTypeMapper.mapTypes(_defineProperty({}, gqlObjectType.name, gqlObjectType));

      return gqlObjectType;
    };

    this.node = function (modelName, related) {
      var opts = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      if (!opts.name) {
        opts.name = '';
      }

      var connection = opts.connection || {};

      var models = _this.thinky.models;

      var modelTarget = models[modelName],
          relation = related;

      if (!modelTarget) {
        throw new Error('Model ' + modelName + ' not found.');
      }

      // Relation is specified as a string
      if (typeof related === 'string') {
        relation = modelTarget._joins[related];
        relation.parentModelName = modelName;

        // relation not found can't continue
        if (!relation) {
          throw new Error('Tried to access relation ' + related + ' of the Model ' + modelName + ',\n             but relation not found.');
        }
      }

      modelTarget = relation ? relation.model : modelTarget;

      return new _node2.default({
        name: related || opts.name,
        model: modelTarget,
        related: relation,
        thinky: _this.thinky,
        query: opts.query,
        loadersKey: _this.loadersKey,
        connection: _extends({}, connection)
      });
    };

    this.thinky = thinky;
    this.options = Object.assign(defaultOptions, options);
    this.loadersKey = options.loadersKey || 'loaders';

    var nodeMapper = (0, _nodeDefinition.nodeInterfaceMapper)(thinky.models);

    this.nodeField = nodeMapper.nodeField;
    this.nodeInterface = nodeMapper.nodeInterface;
    this.nodeTypeMapper = nodeMapper.nodeTypeMapper;
  }

  /**
   * Return Model loader
   * object of all models
   * @returns {*}
   */


  _createClass(GraphqlThinky, [{
    key: 'getModelLoaders',
    value: function getModelLoaders() {
      var models = this.thinky.models;
      return Object.keys(models).reduce(function (loadersObj, modelName) {
        var modelLoader = new _modelLoader2.default(models[modelName]);
        loadersObj[modelName] = modelLoader;
        return loadersObj;
      }, {});
    }

    /**
     * Resolve a node
     *
     * @param modelName
     * @param related
     * @param opts
     * @returns {Resolver}
     */


    /**
     * Create a relay connection
     *
     * @param modelName
     * @param related
     * @param opts
     * @returns {Resolver}
     */


    /**
     * Returns a connection definition
     * based on a graphQL type.
     * This is mostly a helper function
     * when constructing connections based
     * on dataloader.
     * @param gqlType
     * @param args
     * @returns {{type, args: {}}}
     */


    /**
     * return a graphQL Object Type
     * definition from a Thinky Model.
     *
     * @param model
     * @param opts
     */


    /**
     * Create a GraphQLObjectType based
     * to a thinky model. Using this helper
     * you'll get free Node type mapping
     * for relay
     *
     * @param model
     * @param opts
     * @returns {GraphQLObjectType}
     */


    /**
     * Create a node
     *
     * @param modelName
     * @param related
     * @param opts
     * @returns {Node}
     */

  }]);

  return GraphqlThinky;
}();

GraphqlThinky.typeMapper = _typeMapper2.default;
exports.default = GraphqlThinky;