'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _queryBuilder = require('./queryBuilder');

var _relay = require('./relay');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Node class
 */

var Node = function () {
  function Node(_ref) {
    var model = _ref.model;
    var _ref$related = _ref.related;
    var related = _ref$related === undefined ? undefined : _ref$related;
    var _ref$args = _ref.args;
    var args = _ref$args === undefined ? {} : _ref$args;
    var _ref$connection = _ref.connection;
    var connection = _ref$connection === undefined ? {} : _ref$connection;
    var _ref$name = _ref.name;
    var name = _ref$name === undefined ? '' : _ref$name;
    var _ref$query = _ref.query;
    var query = _ref$query === undefined ? undefined : _ref$query;
    var _ref$loadersKey = _ref.loadersKey;
    var loadersKey = _ref$loadersKey === undefined ? 'loaders' : _ref$loadersKey;

    _classCallCheck(this, Node);

    (0, _assert2.default)(model, 'You need to provide a thinky Model');

    this.model = model;
    this.related = related;
    this.args = args;
    this.connection = connection;
    this.name = name; // The name will be populated based to AST name if not provided
    this.query = query;
    this.loadersKey = loadersKey;
  }

  /**
   * Resolve node based
   * a rethinkDB query
   *
   * @param thinky
   * @returns {*}
   */


  _createClass(Node, [{
    key: 'queryResolve',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee2() {
        var _this = this;

        var queryResult, Query, _Query;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                this.args.query = this.query;

                queryResult = void 0;

                if (!this.args.list) {
                  _context2.next = 11;
                  break;
                }

                Query = (0, _queryBuilder.buildQuery)(this.model, this.args, this.model._thinky);
                _context2.next = 6;
                return Query.run();

              case 6:
                queryResult = _context2.sent;

                if (!this.args.count) {
                  _context2.next = 9;
                  break;
                }

                return _context2.delegateYield(regeneratorRuntime.mark(function _callee() {
                  var queryCountResult;
                  return regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          _context.next = 2;
                          return (0, _queryBuilder.buildCount)(_this.model, [], null, _this.args);

                        case 2:
                          queryCountResult = _context.sent;


                          queryResult = queryResult.map(function (row) {
                            row.fullCount = queryCountResult;
                            return row;
                          });

                        case 4:
                        case 'end':
                          return _context.stop();
                      }
                    }
                  }, _callee, _this);
                })(), 't0', 9);

              case 9:
                _context2.next = 15;
                break;

              case 11:
                _Query = (0, _queryBuilder.buildQuery)(this.model, _extends({}, this.args, {
                  offset: false
                }), this.model._thinky);
                _context2.next = 14;
                return _Query.nth(0).default(null).run();

              case 14:
                queryResult = _context2.sent;

              case 15:
                return _context2.abrupt('return', queryResult);

              case 16:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function queryResolve() {
        return ref.apply(this, arguments);
      }

      return queryResolve;
    }()

    /**
     * Resolve from tree
     *
     * @param source
     * @returns {*}
     */

  }, {
    key: 'fromParentSource',
    value: function fromParentSource(source) {
      var result = source[this.name];
      return result;
    }

    /**
     * Create a relay connection
     *
     * @returns {{connectionType, edgeType, nodeType, resolveEdge, connectionArgs, resolve}|*}
     */

  }, {
    key: 'connect',
    value: function connect(resolveOptions) {
      /*eslint-disable */
      if (!this.connection.name) throw new Error("Please specify a connection name, before call connect on a Node");
      if (!this.connection.type) throw new Error("Please specify a connection type, before call connect on a Node");
      /*eslint-enable */

      return (0, _relay.resolveConnection)(this, resolveOptions);
    }

    /**
     * Generate data tree
     *
     * @param treeSource
     * @param thinky
     * @returns {Array}
     */

  }, {
    key: 'resolve',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee3(treeSource, context) {
        var result, loaders;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                result = treeSource;

                if (this.isRelated()) {
                  _context3.next = 7;
                  break;
                }

                _context3.next = 4;
                return this.queryResolve();

              case 4:
                result = _context3.sent;
                _context3.next = 13;
                break;

              case 7:
                if (!(this.isRelated() && treeSource)) {
                  _context3.next = 13;
                  break;
                }

                result = this.fromParentSource(treeSource);

                loaders = context[this.loadersKey];

                if (loaders) {
                  _context3.next = 12;
                  break;
                }

                throw new Error('\n          GraphQL-thinky couldn\'t find any loaders set into the context\n          with the key "' + this.loadersKey + '"\n        ');

              case 12:

                if (!result && treeSource && loaders) {
                  result = this.resolveWithLoaders(treeSource, loaders);
                }

              case 13:
                return _context3.abrupt('return', result);

              case 14:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function resolve(_x, _x2) {
        return ref.apply(this, arguments);
      }

      return resolve;
    }()

    /**
     * Resolve with loaders
     * @param treeSource
     * @param loaders
     * @returns {*}
     */

  }, {
    key: 'resolveWithLoaders',
    value: function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee4(treeSource, loaders) {
        var result, FkJoin, _FkJoin;

        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                result = void 0;
                // Resolve with Loaders from the context

                if (!(this.related.type === 'belongsTo')) {
                  _context4.next = 8;
                  break;
                }

                FkJoin = this.related.leftKey;
                _context4.next = 5;
                return loaders[this.related.model._schema._model._name].loadById(treeSource[FkJoin]);

              case 5:
                result = _context4.sent;
                _context4.next = 12;
                break;

              case 8:
                _FkJoin = this.related.leftKey;
                _context4.next = 11;
                return loaders[this.related.parentModelName].related(this.name, treeSource[_FkJoin], this.args);

              case 11:
                result = _context4.sent;

              case 12:
                if (!this.isConnection()) {
                  _context4.next = 14;
                  break;
                }

                return _context4.abrupt('return', result.fromNodeArgs(this.args));

              case 14:
                if (!this.args.list) {
                  _context4.next = 18;
                  break;
                }

                return _context4.abrupt('return', result.toArray());

              case 18:
                return _context4.abrupt('return', result.toObject(this.args));

              case 19:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function resolveWithLoaders(_x3, _x4) {
        return ref.apply(this, arguments);
      }

      return resolveWithLoaders;
    }()

    /**
     * Append args
     *
     * @param args
     */

  }, {
    key: 'appendArgs',
    value: function appendArgs(args) {
      this.args = _extends({}, this.args, args);
    }

    /**
     * Determine if this node is a connection
     *
     * @returns {string|*}
     */

  }, {
    key: 'isConnection',
    value: function isConnection() {
      return this.connection.name && this.connection.type;
    }

    /**
     * Determine if the node is related
     *
     * @returns {boolean}
     */

  }, {
    key: 'isRelated',
    value: function isRelated() {
      return Boolean(this.related);
    }

    /**
     * Get model of the Node
     *
     * @returns {*}
     */

  }, {
    key: 'getModel',
    value: function getModel() {
      return this.model;
    }

    /**
     * Get model Name
     *
     * @return {string}
     */

  }, {
    key: 'getModelName',
    value: function getModelName() {
      return this.model._schema._model._name;
    }
  }]);

  return Node;
}();

exports.default = Node;