'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = resolver;

var _lodash = require('lodash');

var _thinkyExportSchema = require('thinky-export-schema');

var _thinkyExportSchema2 = _interopRequireDefault(_thinkyExportSchema);

var _graphql = require('graphql');

var _simplifyAst = require('./simplifyAst');

var _simplifyAst2 = _interopRequireDefault(_simplifyAst);

var _queryBuilder = require('./queryBuilder');

var _node = require('./node');

var _relay = require('./relay');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 *
 * @param Node
 * @param opts
 * @returns {function(*=, *=, *=, *=)}
 */
function resolver(Node) {
  var _this = this;

  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var before = _ref.before;
  var after = _ref.after;

  var opts = _objectWithoutProperties(_ref, ['before', 'after']);

  if (before === undefined) {
    before = function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(opts) {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                return _context.abrupt('return', opts);

              case 1:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this);
      }));

      return function before(_x2) {
        return ref.apply(this, arguments);
      };
    }();
  }
  if (after === undefined) {
    after = function () {
      var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(opts) {
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                return _context2.abrupt('return', opts);

              case 1:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this);
      }));

      return function after(_x3) {
        return ref.apply(this, arguments);
      };
    }();
  }

  var Model = Node.getModel();
  var modelSchema = (0, _thinkyExportSchema2.default)(Model);
  var modelRelations = Object.keys(modelSchema.relationships);

  /**
   * Resolver GraphQL
   *
   * @param source
   * @param args
   * @param context
   * @param info
   * @constructor
   */
  var Resolver = function () {
    var ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee3(source, args, context, info) {
      var simplyAST, fields, type, connection, findOptions, nodeArgs, result;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:

              Node.name = Node.name || info.fieldName;

              simplyAST = (0, _simplifyAst2.default)(info.fieldASTs[0], info);
              fields = simplyAST.fields;
              type = info.returnType;
              connection = (0, _relay.isConnection)(info.returnType);

              if (connection) {
                simplyAST = (0, _relay.nodeAST)(simplyAST);
                fields = simplyAST.fields;
                type = (0, _relay.nodeType)(type);
              }

              findOptions = (0, _queryBuilder.argsToFindOptions)(args, Model, opts);
              nodeArgs = _extends({
                list: Boolean(type instanceof _graphql.GraphQLList || connection),
                filterQuery: false,
                attributes: [],
                filter: {},
                index: 0,
                offset: opts.maxLimit || 100,
                skip: undefined,
                order: undefined,
                count: undefined
              }, opts, findOptions);

              // Before Resolve is triggered, good for permission checks
              // and manipulating query exec

              _context3.next = 10;
              return before(nodeArgs, source, args, context, _extends({}, info, {
                ast: simplyAST,
                type: type,
                source: source
              }));

            case 10:
              nodeArgs = _context3.sent;


              nodeArgs.attributes = (0, _lodash.uniq)(Object.keys(fields).filter(function (field) {
                return modelRelations.indexOf(field) === -1 && modelSchema.fields.hasOwnProperty(field) && modelSchema.fields[field] !== 'Virtual';
              }).concat(['id']).concat(opts.attributes || []));

              type = type.ofType || type;
              Node.appendArgs(nodeArgs);

              _context3.next = 16;
              return Node.resolve(source, context || {});

            case 16:
              result = _context3.sent;
              _context3.next = 19;
              return after(result, source, args, context, _extends({}, info, {
                ast: simplyAST,
                type: type,
                source: source
              }));

            case 19:
              return _context3.abrupt('return', _context3.sent);

            case 20:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, _this);
    }));

    return function Resolver(_x4, _x5, _x6, _x7) {
      return ref.apply(this, arguments);
    };
  }();

  return Resolver;
}